[pve-devel] [PATCH qemu-server v2 14/15] api: migrate_vm: use volume content type assertion helpers

Daniel Kral d.kral at proxmox.com
Tue Feb 11 17:08:13 CET 2025


Make use of the newly introduced content type assertion helpers in the
preconditions check paths and the call to `vdisk_alloc` in the API
handler for migrating VMs (and its subcommands and `mtunnel` endpoint).

Since the preconditions existed before, adding a content type safeguard
at `vdisk_alloc` does not break the API.

Signed-off-by: Daniel Kral <d.kral at proxmox.com>
---
changes since v1:
- preserve `check_storage_access_migrate` and make it a wrapper instead
  of replacing each occurrence with the new helper

 PVE/API2/Qemu.pm   |  8 ++------
 PVE/QemuMigrate.pm | 16 +++++++---------
 PVE/QemuServer.pm  | 19 ++++++++++---------
 3 files changed, 19 insertions(+), 24 deletions(-)

diff --git a/PVE/API2/Qemu.pm b/PVE/API2/Qemu.pm
index ea164eea..52e6a3d7 100644
--- a/PVE/API2/Qemu.pm
+++ b/PVE/API2/Qemu.pm
@@ -244,13 +244,9 @@ my $check_storage_access_clone = sub {
 my $check_storage_access_migrate = sub {
     my ($rpcenv, $authuser, $storecfg, $storage, $node) = @_;
 
-    PVE::Storage::storage_check_enabled($storecfg, $storage, $node);
-
     $rpcenv->check($authuser, "/storage/$storage", ['Datastore.AllocateSpace']);
-
-    my $scfg = PVE::Storage::storage_config($storecfg, $storage);
-    die "storage '$storage' does not support vm images\n"
-	if !$scfg->{content}->{images};
+    PVE::Storage::storage_check_enabled($storecfg, $storage, $node);
+    PVE::Storage::assert_content_type_supported($storecfg, $storage, 'images', $node);
 };
 
 my $import_from_volid = sub {
diff --git a/PVE/QemuMigrate.pm b/PVE/QemuMigrate.pm
index ed5ede30..827e54b7 100644
--- a/PVE/QemuMigrate.pm
+++ b/PVE/QemuMigrate.pm
@@ -158,15 +158,13 @@ sub target_storage_check_available {
     my ($self, $storecfg, $targetsid, $volid) = @_;
 
     if (!$self->{opts}->{remote}) {
-	# check if storage is available on target node
-	my $target_scfg = PVE::Storage::storage_check_enabled(
-	    $storecfg,
-	    $targetsid,
-	    $self->{node},
-	);
-	my ($vtype) = PVE::Storage::parse_volname($storecfg, $volid);
-	die "$volid: content type '$vtype' is not available on storage '$targetsid'\n"
-	    if !$target_scfg->{content}->{$vtype};
+	# check if storage is available on target node and supports the volume's content type
+	eval {
+	    PVE::Storage::storage_check_enabled($storecfg, $targetsid, $self->{node});
+	    my ($vtype) = PVE::Storage::parse_volname($storecfg, $volid);
+	    PVE::Storage::assert_content_type_supported($storecfg, $targetsid, $vtype);
+	};
+	die "$volid: $@" if $@;
     }
 }
 
diff --git a/PVE/QemuServer.pm b/PVE/QemuServer.pm
index 7025501f..6678e2f7 100644
--- a/PVE/QemuServer.pm
+++ b/PVE/QemuServer.pm
@@ -2541,14 +2541,13 @@ sub check_storage_availability {
 	my ($sid, $volname) = PVE::Storage::parse_volume_id($volid, 1);
 	return if !$sid;
 
-	# check if storage is available on both nodes
-	my $scfg = PVE::Storage::storage_check_enabled($storecfg, $sid);
-	PVE::Storage::storage_check_enabled($storecfg, $sid, $node);
-
-	my ($vtype) = PVE::Storage::parse_volname($storecfg, $volid);
-
-	die "$volid: content type '$vtype' is not available on storage '$sid'\n"
-	    if !$scfg->{content}->{$vtype};
+	# check if storage is available on both nodes and supports the volume's content type
+	eval {
+	    PVE::Storage::storage_check_enabled($storecfg, $sid);
+	    PVE::Storage::storage_check_enabled($storecfg, $sid, $node);
+	    PVE::Storage::assert_volume_type_supported($storecfg, $volid);
+	};
+	die "$volid: $@" if $@;
    });
 }
 
@@ -5473,7 +5472,9 @@ sub vm_migrate_alloc_nbd_disks {
 	$format = $defFormat if !$format || !grep { $format eq $_ } $validFormats->@*;
 
 	my $size = $drive->{size} / 1024;
-	my $newvolid = PVE::Storage::vdisk_alloc($storecfg, $storeid, $vmid, $format, $size);
+	my $newvolid = PVE::Storage::vdisk_alloc($storecfg, $storeid, $vmid, $format, $size, {
+	    vtype => 'images',
+	});
 	my $newdrive = $drive;
 	$newdrive->{format} = $format;
 	$newdrive->{file} = $newvolid;
-- 
2.39.5





More information about the pve-devel mailing list